#!/usr/bin/env python

import roslib; roslib.load_manifest('harkviz')
import rospy
import threading
import time
from hark_msgs.msg import HarkFFT
from hark_msgs.msg import HarkFFTVal

from pylab import *
ion()

node_name = 'HarkFFT_EachFFTPlotter'
topic_name = 'HarkFFT'
disp_col_num = 1000
disp_interval = 1.0
cb_count = 0
zero_ignore = 0
plot_data = []

# Command line arguments

for cnt in  range(len(sys.argv)) :
    if sys.argv[cnt] == '-h' or sys.argv[cnt] == '--help' :
        print '----------------------------------'
        print ''
        print 'USAGE : '
        print 'rosrun harkviz HarkFFT_EachFFTPlotter.py -c INT -i FLOAT[sec] -t TopicName -n NodeName -z'
        print ''
        print '----------------------------------'
        sys.exit()
    if sys.argv[cnt] == '-c' or sys.argv[cnt] == '--column' :
        try: 
            disp_col_num = int(sys.argv[cnt + 1])
        except ValueError :
            print 'Error : invalid -c INT'
            sys.exit()
    if sys.argv[cnt] == '-i' or sys.argv[cnt] == '--interval' :
        try: 
            disp_interval = float(sys.argv[cnt + 1])
        except ValueError :
            print 'Error : invalid -i FLOAT'
            sys.exit()
    if sys.argv[cnt] == '-t' or sys.argv[cnt] == '--topic' :
        if '-' == sys.argv[cnt + 1][0] :
            print 'Error : invalid topic name'
            sys.exit()
        topic_name = sys.argv[cnt + 1]
    if sys.argv[cnt] == '-n' or sys.argv[cnt] == '--node' :
        if '-' == sys.argv[cnt + 1][0] :
            print 'Error : invalid topic name'
            sys.exit()
        node_name = sys.argv[cnt + 1]
    if sys.argv[cnt] == '-z' or sys.argv[cnt] == '--zero-ignore' :
        zero_ignore = 1

# Plotting

def callback(data):        

    global plot_data
    global disp_col_num
    global cb_count
    global zero_ignore
    nzero_flag = 0

    for i in range(len(data.src)):
        for j in range(len(data.src[i].fftdata_real)):
            if data.src[i].fftdata_real[j] != 0.0 or data.src[i].fftdata_imag[j] != 0.0 :
                nzero_flag = 1

    if nzero_flag == 0 and zero_ignore == 1 :
        return    

    for win in range(len(data.src)):

        new_frame = zeros( (len(data.src[win].fftdata_real), 1) )

        for j in range(len(data.src[win].fftdata_real)):
            real_val = data.src[win].fftdata_real[j]
            imag_val = data.src[win].fftdata_imag[j]
            new_frame[j][0] = log(sqrt(real_val * real_val + imag_val * imag_val))
                
        if cb_count == 0:
            plot_data.append(new_frame)
        else:
            plot_data[win] = hstack( (plot_data[win],new_frame) )
            if plot_data[win].shape[1] >= disp_col_num :
                plot_data[win] = plot_data[win][:,plot_data[win].shape[1]-disp_col_num:]
          
    cb_count = cb_count + 1
    print cb_count

# Plotting

class plotthread(threading.Thread):

    global plot_data
    global disp_interval

    def __init__(self):
        threading.Thread.__init__(self)
        self.setDaemon(True)

    def run(self):
        print "Plotting Start."
        while True :
            for win in range(len(plot_data)):
                pltcnt = 100 * len(plot_data) + 10 + win + 1
                if plot_data[win].shape[0] >= 1 and plot_data[win].shape[1] >= 1 : 
                    imshow(plot_data[win])
                    draw()    
                subplot(pltcnt)
            time.sleep(disp_interval)
                    
# Main

def listener():
    global node_name
    global topic_name
    rospy.init_node(node_name, anonymous=True)
    rospy.Subscriber(topic_name, HarkFFT, callback, None, 100)
    print "Plotter ready."
    rospy.spin()

if __name__ == '__main__':
    t = plotthread()
    t.start()
    listener()

